import { PromiseDelegate } from '@lumino/coreutils';
import * as vega from 'vega';
import * as dataflow from 'vega-dataflow';
import { client } from 'jupyter-jaeger';
/**
 * Tries parsing all string values as dates.  Any that cannot be parsed are left alone
 *
 * https://github.com/Quansight/jupyterlab-omnisci/pull/85#issuecomment-519656950
 */
function parseDates(o) {
    const n = {};
    for (const key in o) {
        const value = o[key];
        let parsed;
        if (typeof value === 'string') {
            parsed = Date.parse(o[key]);
            if (isNaN(parsed)) {
                parsed = value;
            }
        }
        else {
            parsed = value;
        }
        n[key] = parsed;
    }
    return n;
}
async function getData(parameters, abortSignal) {
    const { tracing, kernel } = QueryIbis;
    if (!kernel) {
        throw new Error('Not connected to any kernel');
    }
    const spanExtract = tracing
        ? await client.startSpanExtract({
            name: 'transform',
            relationship: 'follows_from',
            reference: parameters.span
        })
        : null;
    const cleanup = async () => {
        if (tracing) {
            await client.finishSpan(spanExtract);
        }
    };
    if (abortSignal.aborted) {
        await cleanup();
        return null;
    }
    // Fetch the query results from the kernel.
    const comm = kernel.createComm('queryibis');
    const resultPromise = new PromiseDelegate();
    comm.onMsg = msg => resultPromise.resolve(msg.content.data);
    // set span inside comm to be this comm message instead of root span
    if (tracing) {
        parameters = Object.assign(Object.assign({}, parameters), { span: await client.injectSpan(spanExtract) });
    }
    if (abortSignal.aborted) {
        await cleanup();
        return null;
    }
    await comm.open(parameters).done;
    if (abortSignal.aborted) {
        await cleanup();
        return null;
    }
    const result = await resultPromise.promise;
    if (abortSignal.aborted) {
        await cleanup();
        return null;
    }
    const parsedResult = result.map(parseDates);
    await cleanup();
    if (abortSignal.aborted) {
        return null;
    }
    return parsedResult;
}
const TRANSFORM = 'queryibis';
/**
 * Generates a function to query data from an OmniSci Core database.
 * @param {object} params - The parameters for this operator.
 *
 * Inspired by load transform
 * https://github.com/vega/vega/blob/master/packages/vega-transforms/src/Load.js
 */
class QueryIbis extends dataflow.Transform {
    constructor(params) {
        super([], params);
        this._state = { state: "initial" /* initial */ };
    }
    transform(parameters, pulse) {
        if (this._state.state === "fetched" /* fetched */) {
            // update state and return pulse
            return this.output(pulse, this._state.data);
        }
        if (this._state.state === "fetching" /* fetching */) {
            this._state.aborted = true;
        }
        this._state = { state: "fetching" /* fetching */, aborted: false };
        // return promise for non-blocking async loading
        const p = getData(parameters, this._state).then(res => {
            if (res) {
                this._state = { state: "fetched" /* fetched */, data: res };
                return (df) => df.touch(this);
            }
            /* tslint:disable:no-empty */
            return () => { };
        });
        return { async: p };
    }
    /**
     * Copied from
     * https://github.com/vega/vega/blob/d5b979955f67c9557b97eb5ddebb3fef48fe736c/packages/vega-transforms/src/Load.js#L52-L59
     */
    output(pulse, data) {
        data.forEach(dataflow.ingest);
        // tslint:disable-next-line:no-bitwise
        const out = pulse.fork(pulse.NO_FIELDS & pulse.NO_SOURCE);
        out.rem = this.value;
        this.value = out.source = out.add = data;
        this._state = { state: "initial" /* initial */ };
        return out;
    }
}
/**
 * The definition for the transform. Used by the vega dataflow logic
 * to decide how to use the transform.
 */
/* tslint:disable-next-line */
QueryIbis.Definition = {
    type: 'QueryIbis',
    metadata: { changes: true, source: true },
    params: [
        {
            name: 'name',
            type: 'string',
            required: true
        },
        {
            name: 'data',
            type: 'expr',
            required: false
        },
        {
            name: 'transform',
            type: 'transform',
            array: true,
            required: false
        },
        {
            name: 'span',
            type: 'object',
            required: false
        }
    ]
};
export default QueryIbis;
// Get the vega singleton and add our custom transform to it.
vega.transforms[TRANSFORM] = QueryIbis;
